/*
 *    Qizx Free_Engine-4.4p2
 *
 *    This code is part of the Qizx application components
 *    Copyright (c) 2004-2010 Axyana Software -- All rights reserved.
 *
 *    For conditions of use, see the accompanying license files.
 */
package com.qizx.server.api.admin;

import com.qizx.api.QizxException;
import com.qizx.server.util.QizxDriver;
import com.qizx.server.util.QizxRequestBase;
import com.qizx.server.util.RequestException;

import java.io.IOException;

/**
 * Utility request: close and open the database.
 */
public class ServerControlRequest extends QizxRequestBase
{
    private static final String CMD_STATUS = "status";
    private static final String CMD_START = "online";
    private static final String CMD_STOP = "offline";
    private static final String CMD_RELOAD = "reload";
    private static final String CMD_NODES = "nodes";

    public String getName()
    {
        return "server";
    }

    public void handleGet() // status only
        throws RequestException, IOException
    {
        handle(false);
    }

    public void handlePost()
        throws RequestException, IOException
    {
        handle(true);
    }
    
    public void handle(boolean isPost)
            throws RequestException, IOException
    {
        driver = requiredQizxDriver();
        checkAdminRole(driver);
        String command = getParameter("command", CMD_STATUS);
        
        // parameter 'nodes': by default in cluster mode the request is forwarded
        //  to master; this allows to specify the target nodes: 'all' 'master' etc
        String targetNodes = getParameter("target", "master");

//        if (driver.isDispatcher() && "master".equals(targetNodes)) {
//            if (isPost)
//                driver.clusterPost(this, "*");
//            else
//                driver.clusterGet(this);
//            return;
//        }

        try {

            if(!CMD_STATUS.equals(command))
                checkAdminRole(driver);

            response.setContentType(MIME_PLAIN_TEXT);
            if(CMD_START.equals(command)) {
                driver.start();
            }
            else if(CMD_STOP.equals(command)) {
                driver.stop();
            }
            else if(CMD_RELOAD.equals(command)) {
                driver.reload(servlet);
            }
            else if(CMD_NODES.equals(command)) {
                String name = driver.getClusterName();
                if (name != null)
                    println(name);
//                List<ClusterConnector.Host> hosts = driver.getClusterHosts();
//                for(ClusterConnector.Host h : hosts) {
//                    println(h.hostRole + '\t' +
//                            h.hostAddress + '\t' +
//                            h.hostPort + '\t' +
//                            h.load + '\t' +
//                            h.states);
//                }
                return;
            }
            
            println(getStatus(driver));
        }
        catch (QizxException e) {
            throw new RequestException(e);
        }
    }

    private String getStatus(QizxDriver qizxd)
    {
        return qizxd.isRunning() ? "online" : "offline";
    }
}
